package cn.com.qmth.stmms.api.controller.admin;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.bean.ExamReport;
import cn.com.qmth.stmms.biz.exam.bean.ExamView;
import cn.com.qmth.stmms.biz.exam.bean.ExamVo;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.query.ExamSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.mark.model.PictureConfigItem;
import cn.com.qmth.stmms.biz.school.model.School;
import cn.com.qmth.stmms.biz.school.service.SchoolService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.ExamStatus;
import cn.com.qmth.stmms.common.enums.ExamType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.ObjectiveStatus;
import cn.com.qmth.stmms.common.enums.TrackCountPolicy;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;

@Controller("adminExamController")
@RequestMapping("/api/admin/exam")
public class ExamController extends BaseApiController {

    protected static final Logger log = LoggerFactory.getLogger(ExamController.class);

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private SchoolService schoolService;

    @Autowired
    private ExamSubjectService examSubjectService;

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    private ExamService examService;

    // @Autowired
    // private SchoolService schoolService;

    // private Exam validateExam(ApiUser au, Integer examId, ExamType... types)
    // {
    // Exam exam = examService.findById(examId);
    // if (exam == null || !validateSchool(au.getSchoolId(), exam.getSchoolId())
    // || exam.getStatus() != ExamStatus.START) {
    // throw ApiException.EXAM_NOT_ACCESSIBLED;
    // }
    // if (types != null && types.length > 0 &&
    // !Arrays.asList(types).contains(exam.getType())) {
    // throw ApiException.QUERY_PARAM_ERROR.replaceMessage("exam type invalid");
    // }
    // return exam;
    // }

    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<Exam> list(ExamSearchQuery query) {
        ApiUser user = getApiUser();
        query.setSchoolId(user.getSchoolId());
        query.orderByIdDesc();
        query = examService.findByQuery(query);
        return PageUtil.of(query);
    }

    @RequestMapping(value = "types", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamType> examTypeList() {
        return Arrays.asList(ExamType.values());
    }

    @RequestMapping(value = "status", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamStatus> statusLlist() {
        return Arrays.asList(ExamStatus.values());
    }

    @ResponseBody
    @RequestMapping(value = "find", method = RequestMethod.POST)
    public ExamVo find(@RequestParam Integer examId) {
        Exam exam = examService.findById(examId);
        return ExamVo.of(exam);
    }

    @ResponseBody
    @Logging(menu = "创建考试", type = LogType.ADD)
    @RequestMapping(value = "save", method = RequestMethod.POST)
    public void save(@RequestBody ExamVo vo) {
        User user = getApiUser().getUser();
        Exam exam = ExamVo.of(vo);
        exam.setSchoolId(user.getSchoolId());
        exam.setCreatorId(user.getId());
        exam.setStatus(ExamStatus.START);
        exam.setTrackCountPolicy(TrackCountPolicy.LE);
        exam.setObjectiveStatus(ObjectiveStatus.WAITING);
        if (exam.getType().equals(ExamType.MULTI_MEDIA)) {
            exam.setForceSpecialTag(false);
        }
        JSONObject sasConfig = new JSONObject();
        sasConfig.accumulate("passScore", vo.getPassScore());
        sasConfig.accumulate("excellentScore", vo.getExcellentScore());
        exam.setSasConfig(sasConfig.toString());
        exam = examService.save(exam);
    }

    @ResponseBody
    @Logging(menu = "编辑考试", type = LogType.UPDATE)
    @SuppressWarnings("unchecked")
    @RequestMapping(value = "update", method = RequestMethod.POST)
    public void edit(@RequestBody ExamVo exam) {
        User user = getApiUser().getUser();
        Exam oldExam = examService.findById(exam.getId());
        if (oldExam != null && oldExam.getSchoolId().equals(user.getSchoolId())) {
            oldExam.setName(exam.getName());
            oldExam.setExamTime(exam.getExamTime());
            oldExam.setDescription(exam.getDescription());
            oldExam.setStatus(exam.getStatus());
            oldExam.setForceSpecialTag(exam.getForceSpecialTag());
            oldExam.setForbiddenInfo(exam.getForbiddenInfo());
            if (exam.getForbiddenInfo()) {
                oldExam.setForbiddenScore(exam.getForbiddenScore());
            } else {
                oldExam.setForbiddenScore(false);
            }
            oldExam.setEnableSplit(exam.getEnableSplit());
            oldExam.setAutoScroll(exam.getAutoScroll());
            oldExam.setShowReject(exam.getShowReject());
            JSONObject sasConfig = new JSONObject();
            sasConfig.accumulate("passScore", exam.getPassScore());
            sasConfig.accumulate("excellentScore", exam.getExcellentScore());
            oldExam.setSasConfig(sasConfig.toString());
            if (StringUtils.isNotBlank(exam.getSheetConfig())) {
                String sheetConfig = exam.getSheetConfig();
                JSONArray array = JSONArray.fromObject(sheetConfig);
                List<PictureConfigItem> list = JSONArray.toList(array, new PictureConfigItem(), new JsonConfig());
                oldExam.setSheetConfig(list == null ? null : StringUtils.join(list, PictureConfigItem.DB_ITEM_JOINER));
            }
            oldExam.setStartTime(exam.getStartTime());
            oldExam.setEndTime(exam.getEndTime());
            oldExam.setMarkMode(exam.getMarkMode());
            oldExam.setSheetView(exam.getSheetView());
            oldExam.setInspectScrollBottom(exam.getInspectScrollBottom());
            oldExam.setInspectUnrepeated(exam.getInspectUnrepeated());
            oldExam.setShowObjectiveScore(exam.getShowObjectiveScore());
            oldExam.setRemarkCount(exam.getRemarkCount());
            oldExam.setInspectRoundLimit(exam.getInspectRoundLimit());
            oldExam.setTrackCountPolicy(exam.getTrackCountPolicy());
            examService.save(oldExam);
        }
    }

    @ResponseBody
    @Logging(menu = "考试详情", type = LogType.QUERY)
    @RequestMapping(value = "detail", method = RequestMethod.POST)
    public ExamView view(@RequestParam Integer examId) {
        ExamView ret = new ExamView();
        Exam exam = examService.findById(examId);
        long studentCount = examStudentService.countByExamId(exam.getId());
        long subjectCount = examSubjectService.count(examId);
        // long markerCount = markerService.countByExam(exam.getId());
        long scanCount = examStudentService.countByExamIdAndUpload(examId, true);
        long markedCount = examStudentService.countByExamIdAndSubjectiveScoreListIsNotNull(examId);
        long sheetCount = studentCount == 0 ? 0 : examStudentService.countSheetCountByExamId(examId);

        ret.setStudentCount(studentCount);

        ret.setSubjectCount(subjectCount);
        ret.setScanCount(scanCount);

        long sheetCount2 = new BigDecimal(sheetCount).divide(new BigDecimal(2), 0, BigDecimal.ROUND_HALF_UP)
                .longValue();

        ret.setPaperCount(sheetCount2);

        ret.setMarkedCount(markedCount);

        return ret;
    }

    @ResponseBody
    @RequestMapping(value = "report", method = RequestMethod.POST)
    public ExamReport report(@RequestParam Integer examId) {
        ExamReport ret = new ExamReport();
        Exam exam = examService.findById(examId);
        School school = schoolService.findById(exam.getSchoolId());
        ret.setSchoolName(school.getName());
        ret.setYear(Calendar.getInstance().get(Calendar.YEAR));
        ret.setSubjectCount(subjectService.count(examId));
        ret.setStudentCount(studentService.countByExamId(examId));
        ret.setSubjectiveStudentCount(studentService.countSubjectiveByExamId(examId));
        long totalPaperCount = studentService.countSheetCountByExamId(examId) / 2;
        ret.setObjectiveStudentCount(studentService.countObjectiveByExamId(examId));
        long objectivePaperCount = studentService.countObjectivePageCountByExamId(examId) / 2;
        ret.setSubjectivePaperCount(totalPaperCount - objectivePaperCount);
        ret.setObjectivePaperCount(objectivePaperCount);
        ret.setAllUploadStudentCount(studentService.countAllUploadByExamId(examId));
        ret.setTotalPaperCount(totalPaperCount);
        return ret;
    }
}
