package cn.com.qmth.stmms.api.controller.admin;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.MarkStatus;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.service.*;
import cn.com.qmth.stmms.biz.exam.service.query.ExamSubjectSearchQuery;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.report.service.ReportService;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import net.sf.json.JSONObject;

@Controller("adminSubjectController")
@RequestMapping("/api/admin/subject")
public class SubjectController extends BaseApiController {

    protected static final Logger log = LoggerFactory.getLogger(SubjectController.class);

    private static final String NULL_PAPER_TYPE_PLACEHOLDER = "#";

    private static final String[] SUBJECTIVE_EXCEL_HEADER = new String[] { "科目代码*", "科目名称", "大题名称*", "题目昵称",
            "大题号(只能用小写数字)*", "小题号(只能用小写数字)*", "小题满分*", "给分次数", "间隔分*", "评卷分组(只能用小写数字)*", "图片序号(用英文逗号分割)",
            "仲裁方式(0-分组,1-小题)", "双评比例(0~1)", "仲裁阀值", "合分策略(1-平均，2-最高，3-最低)", "评卷模式(common-普通，track-轨迹)", "试评数量(0-跳过试评)",
            "选做题数量" };

    private static final String[] OBJECTIVE_EXCEL_HEADER = new String[] { "科目代码", "科目名称", "试卷类型", "大题名称", "题目昵称",
            "大题号(只能用小写数字)", "小题号(只能用小写数字)", "标准答案", "小题满分", "题型(1-单选，2-多选，3-判断)" };

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamService examService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private FileService fileService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Autowired
    private LockService lockService;

    @Autowired
    private ReportService reportService;

    @Autowired
    private MarkService markService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private SelectiveGroupService selectiveGroupService;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private InspectHistoryService inspectHistoryService;

    @Value("${file.temp}")
    private String tempFile;

    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamSubject> list(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        return getExamSubject(examId, RequestUtils.getApiUser(request));
    }

    @RequestMapping(value = "/level/list", method = RequestMethod.POST)
    @ResponseBody
    public List<String> levelList(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        return subjectService.listLevel(examId);
    }

    @RequestMapping(value = "/category/list", method = RequestMethod.POST)
    @ResponseBody
    public List<String> categoryList(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        return subjectService.listCategory(examId);
    }

    @RequestMapping(value = "/query", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<ExamSubject> list(HttpServletRequest request, ExamSubjectSearchQuery query) {
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query = subjectService.findByQuery(query);
        for (ExamSubject subject : query.getResult()) {
            List<String> paperTypes = questionService.getPaperType(examId, subject.getCode());
            subject.setPaperType(StringUtils.join(paperTypes, ","));
            subject.setPaperAnswerUrl(fileService);
            subject.setLocked(lockService.isLocked(LockType.SCORE_CALCULATE, examId, subject.getCode()));
        }
        return PageUtil.of(query);
    }

    @RequestMapping(value = "/find", method = RequestMethod.POST)
    @ResponseBody
    public ExamSubject find(HttpServletRequest request, @RequestParam Integer examId, @RequestParam String code) {
        return subjectService.find(examId, code);
    }

    @Logging(menu = "科目设置修改", type = LogType.UPDATE)
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject save(RedirectAttributes redirectAttributes, HttpServletRequest request, ExamSubject subject) {
        ExamSubject previous = subjectService.find(getSessionExamId(request), subject.getCode());
        if (previous == null) {
            throw new StatusException("科目不存在");
        }
        previous.setSheetConfig(subject.getSheetConfig());
        previous.setSasConfig(subject.getSasConfig());
        previous.setEnableSplit(subject.getEnableSplit());
        previous.setAutoScroll(subject.getAutoScroll());
        previous.setDisplayQuestionName(
                subject.getDisplayQuestionName() == null ? false : subject.getDisplayQuestionName());
        subjectService.save(previous);
        return result(true);
    }

    @RequestMapping(value = "/statistic", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject statistic(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        List<Double> totalScoreList = subjectService.distinctTotalScore(examId);
        List<Long> subjectCountList = new ArrayList<Long>();
        for (Double totalScore : totalScoreList) {
            ExamSubjectSearchQuery querys = new ExamSubjectSearchQuery();
            querys.setExamId(examId);
            querys.setTotalScore(totalScore);
            subjectCountList.add(subjectService.countByQuery(querys));
        }
        return result(true);
    }
}
